"""Dimensions module.

This is the private implementation of the dimensions module.

"""

__all__ = ["AbstractDimension", "dimension", "dimension_of"]

from .core import AbstractDimension, dimension, dimension_of
